#include "evmc66c_PCI.h"

#include <stdio.h>
#include <ti/sysbios/BIOS.h>

#include "DepUtility.h"
#include <ti/csl/csl_bootcfgAux.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_xmcAux.h>

#include <string.h>

/* In this example all addresses are 32bit */
/* Outbound Base Address for PCIe Master */
#define PCIE_OB_LO_ADDR_M    0x60000000
#define PCIE_OB_HI_ADDR_M    0

/* Inbound  Base Address for PCIe Master */
#define PCIE_IB_LO_ADDR_M    0x90000000
#define PCIE_IB_HI_ADDR_M    0

/* Outbound Base Address for PCIe Slave */
#define PCIE_OB_LO_ADDR_S    PCIE_IB_LO_ADDR_M
#define PCIE_OB_HI_ADDR_S    0

/* Inbound  Base Address for PCIe Slave */
#define PCIE_IB_LO_ADDR_S    PCIE_OB_LO_ADDR_M
#define PCIE_IB_HI_ADDR_S    0

/* BAR mask */
#define PCIE_BAR_MASK         0x0FFFFFFF

/* BAR Index PCie*/
#define PCIE_BAR_IDX_M         1
#define PCIE_BAR_IDX_S         1


/* PCIe Regions used in the example */
#define PCIE_IB_REGION_M    0
#define PCIE_OB_REGION_M    0
#define PCIE_IB_REGION_S    0
#define PCIE_OB_REGION_S    0

/* Size of application buffers */
#define PCIE_BUFSIZE_APP 40






/* last element in the buffer is a marker that indicates the buffer status: full/empty */
#define PCIE_EXAMPLE_MAX_CACHE_LINE_SIZE 128
#define PCIE_EXAMPLE_UINT32_SIZE           4 /* preprocessor #if requires a real constant, not a sizeof() */

#define PCIE_EXAMPLE_DSTBUF_BYTES ((PCIE_BUFSIZE_APP + 1) * PCIE_EXAMPLE_UINT32_SIZE)
#define PCIE_EXAMPLE_DSTBUF_REM (PCIE_EXAMPLE_DSTBUF_BYTES % PCIE_EXAMPLE_MAX_CACHE_LINE_SIZE)
#define PCIE_EXAMPLE_DSTBUF_PAD (PCIE_EXAMPLE_DSTBUF_REM ? (PCIE_EXAMPLE_MAX_CACHE_LINE_SIZE - PCIE_EXAMPLE_DSTBUF_REM) : 0)

/* Cache coherence: Align must be a multiple of cache line size (L2=128 bytes, L1=64 bytes) to operate with cache enabled. */
/* Aligning to 256 bytes because the PCIe inbound offset register masks the last 8bits of the buffer address  */
#pragma DATA_SECTION(".dstBufSec")
#pragma DATA_ALIGN(256)
struct dstBuf_s {
  volatile uint32_t buf[PCIE_BUFSIZE_APP + 1];
  /* Cache coherence: Must pad to cache line size in order to enable cacheability */
#if PCIE_EXAMPLE_DSTBUF_PAD
  uint8_t padding[PCIE_EXAMPLE_DSTBUF_PAD];
#endif
} dstBuf;

#define PCIE_EXAMPLE_BUF_EMPTY 0
#define PCIE_EXAMPLE_BUF_FULL  1

/* Does not need to be aligned (even for cache) since it is only accessed locally */
uint32_t srcBuf[PCIE_BUFSIZE_APP];





extern "C"
{
	#include "ti/platform/platform.h"
	#include "ti/platform/evmc6678l/platform_lib/include/evmc66x_uart.h"

	
	
	/* CSL include */
	#include <ti/csl/csl_pscAux.h>
	#include <ti/csl/cslr_device.h>

}

/*****************************************************************************
 * Function: Converts a core local L2 address to a global L2 address 
 *   Input addr:  L2 address to be converted to global.
 *   return:  uint32_t   Global L2 address
 *****************************************************************************/
uint32_t pcieConvert_CoreLocal2GlobalAddr (uint32_t  addr)
{
  uint32_t coreNum;

  /* Get the core number. */
  coreNum = CSL_chipReadReg(CSL_CHIP_DNUM); 
  
  /* Compute the global address. */
  return ((1 << 28) | (coreNum << 24) | (addr & 0x00ffffff));
}    

/*****************************************************************************
 * Function: Enable/Disable LTSSM (Link Training)
 ****************************************************************************/
pcieRet_e pcieLtssmCtrl(Pcie_Handle handle, uint8_t enable)
{
  pcieCmdStatusReg_t    cmdStatus;
  pcieRegisters_t       setRegs;
  pcieRegisters_t       getRegs;
  pcieRet_e retVal;

  memset (&cmdStatus,    0, sizeof(cmdStatus));
  memset (&setRegs,      0, sizeof(setRegs));
  memset (&getRegs,      0, sizeof(getRegs));

  getRegs.cmdStatus = &cmdStatus;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Read CMD STATUS register failed!\n");
    return retVal;
  }
  
  if(enable)
    cmdStatus.ltssmEn = 1;
  else
    cmdStatus.ltssmEn = 0;

  setRegs.cmdStatus = &cmdStatus;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
	  printf ("SET CMD STATUS register failed!\n");
    return retVal;
  }

  return pcie_RET_OK;
}

void cycleDelay (uint32_t count)
{
  uint32_t sat;

  if (count <= 0)
    return;

  sat = TSCL + count;
  while (TSCL < sat);
}

pcieRet_e pcieSerdesCfg(void)
{
  uint16_t cfg;

  CSL_BootCfgUnlockKicker();

  /* Provide PLL reference clock to SERDES inside PCIESS
     Program PLL settings and enable PLL from PCIe SERDES.*/
  cfg = 0x01C9; /* value based on PCIe userguide */
  CSL_BootCfgSetPCIEConfigPLL(cfg);

  CSL_BootCfgLockKicker();

  /*Wait for PLL to lock (3000 CLKIN1 cycles) */
  cycleDelay(10000);

  return pcie_RET_OK;
}

/*****************************************************************************
 * Function: Power domain configuration
 ****************************************************************************/
pcieRet_e pciePowerCfg(void)
{
  /* Turn on the PCIe power domain */
  if (CSL_PSC_getPowerDomainState (CSL_PSC_PD_PCIEX) != PSC_PDSTATE_ON) {
    /* Enable the domain */
    CSL_PSC_enablePowerDomain (CSL_PSC_PD_PCIEX);
    /* Enable MDCTL */
    CSL_PSC_setModuleNextState (CSL_PSC_LPSC_PCIEX, PSC_MODSTATE_ENABLE);
    /* Apply the domain */
    CSL_PSC_startStateTransition (CSL_PSC_PD_PCIEX);
    /* Wait for it to finish */
    while (! CSL_PSC_isStateTransitionDone (CSL_PSC_PD_PCIEX));
  } else {
    printf ("Power domain is already enabled.  You probably re-ran without device reset (which is OK)\n");
  }

  return pcie_RET_OK;
}

/*****************************************************************************
 * Function: Configure and enable Inbound Address Translation 
 ****************************************************************************/
pcieRet_e pcieIbTransCfg(Pcie_Handle handle, pcieIbTransCfg_t *ibCfg)
{
  pcieRet_e retVal;

  pcieRegisters_t      setRegs;
  pcieRegisters_t      getRegs;

  pcieCmdStatusReg_t   cmdStatus;

  memset (&setRegs,   0, sizeof(setRegs));
  memset (&getRegs,   0, sizeof(getRegs));
  memset (&cmdStatus, 0, sizeof(cmdStatus));

  /* Set outbound offset registers */
  if ((retVal = Pcie_cfgIbTrans(handle, ibCfg)) != pcie_RET_OK) 
  {
    printf ("Failed to configure Inbound Translation registers!\n");
    return retVal;
  }

  /*enable Inbound address translation*/
  memset (&setRegs,    0, sizeof(setRegs));
  memset (&getRegs,    0, sizeof(getRegs));

  getRegs.cmdStatus = &cmdStatus;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
	  printf ("Read CMD STATUS register failed!\n");
    return retVal;

  }
  cmdStatus.ibXltEn = 1;
  setRegs.cmdStatus = &cmdStatus;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
	  printf ("SET CMD STATUS register failed!\n");
    return retVal;
  }

  return pcie_RET_OK;
}

/*****************************************************************************
 * Function: Configure and enable Outbound Address Translation 
 ****************************************************************************/
pcieRet_e pcieObTransCfg(Pcie_Handle handle, uint32_t obAddrLo, uint32_t obAddrHi, uint8_t region)
{

  pcieRet_e retVal;

  pcieRegisters_t      setRegs;
  pcieRegisters_t      getRegs;

  pcieCmdStatusReg_t   cmdStatus;

  memset (&setRegs,   0, sizeof(setRegs));
  memset (&getRegs,   0, sizeof(getRegs));
  memset (&cmdStatus, 0, sizeof(cmdStatus));

  /* Set outbound offset registers */
  if ((retVal = Pcie_cfgObOffset(handle, obAddrLo, obAddrHi, region)) != pcie_RET_OK) 
  {
    printf ("Failed to configure ObOffset registers!\n");
    return retVal;
  }

  /*enable Outbound address translation*/
  memset (&setRegs,    0, sizeof(setRegs));
  memset (&getRegs,    0, sizeof(getRegs));

  getRegs.cmdStatus = &cmdStatus;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Read CMD STATUS register failed!\n");
    return retVal;
  }
  cmdStatus.obXltEn = 1;
  setRegs.cmdStatus = &cmdStatus;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET CMD STATUS register failed!\n");
    return retVal;
  }

  return pcie_RET_OK;
}


evmc66c_PCI::evmc66c_PCI()
{
	
}

evmc66c_PCI::~evmc66c_PCI(void)
{
	
}

int evmc66c_PCI::init(void)
{
	return 0;	
}

unsigned int evmc66c_PCI::readReg(unsigned int offset)
{
#define PCIE_START_ADDRESS	(0x21800000)

	unsigned int temp;
	unsigned int *p;

	p = (unsigned int *)PCIE_START_ADDRESS + offset;

	temp = *p;

	return temp;

}

int  evmc66c_PCI::printRegisters(void)
{
	Pcie_Handle      	handle = NULL;
	pcieRegisters_t     setRegs;
	pcieRegisters_t     getRegs;
	int retVal;

	memset (&setRegs,    0, sizeof(setRegs));
	memset (&getRegs,    0, sizeof(getRegs));

	Pcie_open(0, &handle);

	pciePidReg_t 				pid;
	pcieCmdStatusReg_t 			cmdStatus;
	pciePmRstIrqStatusRawReg_t	pmRstIrqStatusRaw;
	pcieVndDevIdReg_t         	vndDevId;                /**< @brief Vendor and device ID*/
	pcieStatusCmdReg_t          statusCmd;

	  /*Capabilities Registers*/
	  pciePciesCapReg_t                 pciesCap;                /**< @brief PCI Express Capabilities Register*/
	  pcieDeviceCapReg_t                deviceCap;               /**< @brief Device Capabilities Register*/
	  pcieDevStatCtrlReg_t              devStatCtrl;             /**< @brief Device Status and Control*/
	  pcieLinkCapReg_t                  linkCap;                 /**< @brief Link Capabilities Register*/
	  pcieLinkStatCtrlReg_t             linkStatCtrl;            /**< @brief Link Status and Control Register*/
	  pcieSlotCapReg_t                  slotCap;                 /**< @brief Slot Capabilities Register */
	  pcieSlotStatCtrlReg_t             slotStatCtrl;            /**< @brief Slot Status and Control Register */
	  pcieRootCtrlCapReg_t              rootCtrlCap;             /**< @brief Root Control and Capabilities Register */
	  pcieRootStatusReg_t               rootStatus;              /**< @brief Root Status and Control Register */
	  pcieDevCap2Reg_t                  devCap2;                 /**< @brief Device Capabilities 2 Register*/
	  pcieDevStatCtrl2Reg_t             devStatCtrl2;            /**< @brief Device Status and Control 2 Register*/
	  pcieLinkCtrl2Reg_t                linkCtrl2;               /**< @brief Link Control 2 Register*/

	  /*Port Logic Registers*/
	  pciePlAckTimerReg_t               plAckTimer;              /**< @brief Ack Latency Time and Replay Timer */
	  pciePlOMsgReg_t                   plOMsg;                  /**< @brief Other Message */
	  pciePlForceLinkReg_t              plForceLink;             /**< @brief Port Force Link */
	  pcieAckFreqReg_t                  ackFreq;                 /**< @brief Ack Frequency */
	  pcieLnkCtrlReg_t                  lnkCtrl;                 /**< @brief Port Link Control*/
	  pcieLaneSkewReg_t                 laneSkew;                /**< @brief Lane Skew */
	  pcieSymNumReg_t                   symNum;                  /**< @brief Symbol Number */
	  pcieSymTimerFltMaskReg_t          symTimerFltMask;         /**< @brief Symbol Timer and Filter Mask */
	  pcieFltMask2Reg_t                 fltMask2;                /**< @brief Filter Mask 2 */
	  pcieDebug0Reg_t                   debug0;                  /**< @brief Debug 0*/
	  pcieDebug1Reg_t                   debug1;                  /**< @brief Debug 1 Register*/
	  pcieGen2Reg_t                     gen2;                    /**< @brief Gen2 */

	getRegs.pid = &pid;
	getRegs.cmdStatus = &cmdStatus;
	getRegs.pmRstIrqStatusRaw=&pmRstIrqStatusRaw;
	getRegs.vndDevId=&vndDevId;
	getRegs.statusCmd=&statusCmd;
	getRegs.pciesCap =&pciesCap;
	getRegs.deviceCap =&deviceCap;
	getRegs.devStatCtrl =&devStatCtrl;
	getRegs.linkCap =&linkCap;
	getRegs.linkStatCtrl =&linkStatCtrl;
	getRegs.slotCap =&slotCap;
	getRegs.slotStatCtrl =&slotStatCtrl;
	getRegs.rootCtrlCap =&rootCtrlCap;
	getRegs.rootStatus =&rootStatus;
	getRegs.devCap2 =&devCap2;
	getRegs.devStatCtrl2 =&devStatCtrl2;
	getRegs.linkCtrl2 =&linkCtrl2;

	getRegs.plAckTimer=&plAckTimer;
	getRegs.plOMsg=&plOMsg;
	getRegs.plForceLink=&plForceLink;
	getRegs.ackFreq=&ackFreq;
	getRegs.lnkCtrl=&lnkCtrl;
	getRegs.laneSkew=&laneSkew;
	getRegs.symNum=&symNum;
	getRegs.symTimerFltMask=&symTimerFltMask;
	getRegs.fltMask2=&fltMask2;
	getRegs.debug0=&debug0;
	getRegs.debug1=&debug1;
	getRegs.gen2=&gen2;
	if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK)
	{
		printf ("Read register failed!\n");
		return retVal;
	}


	//printf ("PID: 0x%08X\n", pid.raw);
	printf ("CMD_STATUS: 0x%08X\n", cmdStatus.raw);
/*
	printf ("PMRST: 0x%08X\n", pmRstIrqStatusRaw.raw);
	printf ("VENDOR_DEVICE: 0x%08X\n", vndDevId.raw);
	printf ("STATUS_COMMAND: 0x%08X\n", statusCmd.raw);
	printf ("pciesCap: 0x%08X\n", pciesCap.raw);
	printf ("deviceCap: 0x%08X\n", deviceCap.raw);
	printf ("devStatCtrl: 0x%08X\n", devStatCtrl.raw);
	printf ("linkCap: 0x%08X\n", linkCap.raw);
*/
	printf ("linkStatCtrl: 0x%08X\n", linkStatCtrl.raw);
/*
 	printf ("slotCap: 0x%08X\n", slotCap.raw);
	printf ("slotStatCtrl: 0x%08X\n", slotStatCtrl.raw);
	printf ("rootCtrlCap: 0x%08X\n", rootCtrlCap.raw);
	printf ("rootStatus: 0x%08X\n", rootStatus.raw);
	printf ("devCap2: 0x%08X\n", devCap2.raw);
	printf ("devStatCtrl2: 0x%08X\n", devStatCtrl2.raw);
	printf ("linkCtrl2: 0x%08X\n", linkCtrl2.raw);

	printf ("plAckTimer: 0x%08X\n", plAckTimer.raw);
	printf ("plOMsg: 0x%08X\n", plOMsg.raw);
	printf ("plForceLink: 0x%08X\n", plForceLink.raw);
	printf ("ackFreq: 0x%08X\n", ackFreq.raw);
	printf ("lnkCtrl: 0x%08X\n", lnkCtrl.raw);

	printf ("laneSkew: 0x%08X\n", laneSkew.raw);
	printf ("symNum: 0x%08X\n", symNum.raw);
	printf ("symTimerFltMask: 0x%08X\n", symTimerFltMask.raw);
	printf ("fltMask2: 0x%08X\n", fltMask2.raw);
*/
	printf ("debug0: 0x%08X\n", debug0.raw);
	printf ("debug1: 0x%08X\n", debug1.raw);
	//printf ("gen2: 0x%08X\n", gen2.raw);

/*
	if((cmdStatus.ltssmEn) == 0)
	{
		printf ("s\n");
		this->start();


	}
	*/

/*
#define PCIE_PID_OFFSET			(0x00000000)
#define PCIE_CMD_STATUS_OFFSET	(0x00000004)
#define PCIE_CFG_SETUP_OFFSET	(0x00000008)
#define PCIE_PMRST_OFFSET		(0x000001D0)
#define PCIE_VENDOR_DEVICE_OFFSET	(0x00001000)
#define PCIE_STATUS_COMMAND_OFFSET	(0x00001004)


	unsigned int temp;

	printf ("\n\n");

	temp = readReg(PCIE_PID_OFFSET);
	printf ("PID: 0x%08X\n", temp);

	temp = readReg(PCIE_CMD_STATUS_OFFSET);
	printf ("CMD_STATUS: 0x%08X\n", temp);

	temp = readReg(PCIE_CFG_SETUP_OFFSET);
	printf ("CFG_SETUP: 0x%08X\n", temp);

	temp = readReg(PCIE_PMRST_OFFSET);
	printf ("PMRST: 0x%08X\n", temp);

	temp = readReg(PCIE_VENDOR_DEVICE_OFFSET);
	printf ("VENDOR_DEVICE: 0x%08X\n", temp);

	temp = readReg(PCIE_STATUS_COMMAND_OFFSET);
	printf ("STATUS_COMMAND: 0x%08X\n", temp);
*/
	return 0;
}

/*****************************************************************************
 * Function: Check LTSSM status and wait for the link to be up
 ****************************************************************************/
void evmc66c_PCI::pcieWaitLinkUp(Pcie_Handle handle)
{
  pcieRegisters_t  getRegs;
  pcieDebug0Reg_t  debug0;

  memset (&debug0,  0, sizeof(debug0));
  memset (&getRegs, 0, sizeof(getRegs));

  uint8_t ltssmState = 0;

  while(ltssmState != pcie_LTSSM_L0)
  {
    cycleDelay(100);
    getRegs.debug0 = &debug0;
    if (Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs) != pcie_RET_OK)
    {
      printf ("Read LTSSM state failed!\n");
      return;
    }
    ltssmState = debug0.ltssmState;
  }
}

int evmc66c_PCI::start(void)
{
	int ret;
	int retVal;
	Pcie_Handle      handle = NULL;
	pcieIbTransCfg_t ibCfg;
  	pcieBarCfg_t     barCfg;

	
	printf ("Version #: 0x%08x; string %s\n\n", Pcie_getVersion(), Pcie_getVersionStr());
	
	/* Configure SERDES*/
	if ((ret = pcieSerdesCfg()) != pcie_RET_OK)	printf ("PCIe Serdes config failed (%d)\n", (int)ret);
	else														printf ("PCIe Serdes config OK \n");

	
	
	/* Power up PCIe Module */
	if ((ret = pciePowerCfg()) != pcie_RET_OK) 
	{
	  printf ("PCIe Power Up failed (%d)\n", (int)ret);
	  return -1;
	}
	
	printf ("PCIe Power Up.\n");
	
	/* Set the PCIe mode*/
		if ((ret = Pcie_setMode(pcie_EP_MODE)) != pcie_RET_OK)	printf("Set PCIe Mode failed (%d)\n", (int)ret);
		else 																		printf("Set PCIe Mode OK\n");


	/* Wait until the PCIe SERDES PLL locks */
	uint16_t lock=0;
	while (!lock)
	{
	  CSL_BootCfgGetPCIEPLLLock(&lock);
	}
	
	if ((ret = Pcie_open(0, &handle)) != pcie_RET_OK)
	{
	  printf ("Open failed (%d)\n", (int)ret);
	  return -2;
	
	}
	
	
	if((ret = pcieCfgEP(handle)) != pcie_RET_OK) 
   {
      printf ("Failed to configure PCIe in EP mode (%d)\n", (int)ret);
      return -3;
   }
   else printf ("PLL configured.\n");
	
	/**/
	if((ret = pcieCustomCfgEp(handle)) != pcie_RET_OK)
   {
      printf ("Failed to custom configure PCIe in EP mode (%d)\n", (int)ret);
      return -3;
   }
   else printf ("Custom configuration OK.\n");


	/*enable link training*/
	/*
	  if ((retVal = pcieLtssmCtrl(handle, TRUE)) != pcie_RET_OK)
	  {
	    printf ("Failed to enable Link Training!\n");
	    return -1;
	  }

*/

	/* Configure Address Translation */
    
    barCfg.location = pcie_LOCATION_LOCAL;
    barCfg.mode     = pcie_EP_MODE;
    barCfg.base     = PCIE_IB_LO_ADDR_S;
    barCfg.prefetch = pcie_BAR_NON_PREF;
    barCfg.type     = pcie_BAR_TYPE32;
    barCfg.memSpace = pcie_BAR_MEM_MEM;
    barCfg.idx      = PCIE_BAR_IDX_S;
    
    if ((retVal = Pcie_cfgBar(handle, &barCfg)) != pcie_RET_OK) 
    {
      printf ("Failed to configure BAR!\n");
      return -4;
     
    }

    ibCfg.ibBar         = PCIE_BAR_IDX_S; /* Match BAR that was configured above*/
    ibCfg.ibStartAddrLo = PCIE_IB_LO_ADDR_S;
    ibCfg.ibStartAddrHi = PCIE_IB_HI_ADDR_S;
    ibCfg.ibOffsetAddr  = (uint32_t)pcieConvert_CoreLocal2GlobalAddr ((uint32_t)dstBuf.buf);
    ibCfg.region        = PCIE_IB_REGION_S;       

    if ((retVal = pcieIbTransCfg(handle, &ibCfg)) != pcie_RET_OK) 
    {
      printf ("Failed to configure Inbound Translation!\n", (int)retVal);
      return -5;
      
    }
    else
    {
      printf ("Successfully configured Inbound Translation!\n");
    }

    if ((retVal = pcieObTransCfg (handle, PCIE_OB_LO_ADDR_S, PCIE_OB_HI_ADDR_S, PCIE_OB_REGION_S)) != pcie_RET_OK) 
    {
      printf ("Failed to configure Outbound Address Translation!\n", (int)retVal);
      return -6;
     
    }
    else
    {
      printf ("Successfully configured Outbound Translation!\n");
    }

    printf ("Starting link training...\n");

      /*Enable link training*/
      if ((retVal = pcieLtssmCtrl(handle, TRUE)) != pcie_RET_OK)
      {
    	  printf ("Failed to Enable Link Training!\n", (int)retVal);
        exit(1);
      }

      /* Wait for link to be up */
      //pcieWaitLinkUp(handle);

      //printf ("Link is up.\n");


	return 0;	
}


int evmc66c_PCI::pcieCustomCfgEp(Pcie_Handle handle)
{
	pcieRegisters_t     regs;
	int retVal;

	pcieLnkCtrlReg_t                  lnkCtrl;                 /**< @brief Port Link Control*/


	memset (&regs,    0, sizeof(regs));

	//Lettura registri
	regs.lnkCtrl=&lnkCtrl;
	if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &regs)) != pcie_RET_OK)
	{
		printf ("Read register failed!\n");
		return retVal;
	}

	// PCIe 1x
	lnkCtrl.lnkMode = 0x1; //1x



	//Scrittura registri
	if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &regs)) != pcie_RET_OK)
	{
		printf ("Write register failed!\n");
		return retVal;
	}


	return 0;
}
/*****************************************************************************
 * Function: Configure PCIe in End Point Mode
 ****************************************************************************/
int evmc66c_PCI::pcieCfgEP(Pcie_Handle handle)
{
  pcieRet_e retVal;

  pcieCmdStatusReg_t     cmdStatus;
  pcieObSizeReg_t        obSize;
  pcieGen2Reg_t          gen2;
  pcieType0Bar32bitIdx_t type0Bar32bitIdx;
  pcieStatusCmdReg_t     statusCmd;
  pcieDevStatCtrlReg_t   devStatCtrl;
  pcieAccrReg_t          accr;
                 
  pcieRegisters_t        setRegs;
  pcieRegisters_t        getRegs;

  memset (&cmdStatus,        0, sizeof(cmdStatus));
  memset (&obSize,           0, sizeof(obSize));
  memset (&gen2,             0, sizeof(gen2));
  memset (&type0Bar32bitIdx, 0, sizeof(type0Bar32bitIdx));
  memset (&statusCmd,        0, sizeof(statusCmd));
  memset (&devStatCtrl,      0, sizeof(devStatCtrl));
  memset (&accr,             0, sizeof(accr));

  /*Disable link training*/
  if ((retVal = pcieLtssmCtrl(handle, FALSE)) != pcie_RET_OK) 
  {
    printf ("Failed to disable Link Training!\n");
    return -1;
  }
  
  /* Configure the size of the translation regions */   
  memset (&setRegs, 0, sizeof(setRegs));
  memset (&getRegs, 0, sizeof(getRegs));
  
  obSize.size = pcie_OB_SIZE_8MB;
  setRegs.obSize = &obSize;
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET OB_SIZE register failed!\n");
    return -2;
  }
     
  /* Setting PL_GEN2 */   
  memset (&setRegs, 0, sizeof(setRegs));
  gen2.numFts = 0xF;
  gen2.dirSpd = 0x0;
  gen2.lnEn   = 1;
  setRegs.gen2 = &gen2;
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET GEN2 register failed!\n");
    return -3;
  }
  
  /* Configure BAR Masks */   
  /* First need to enable writing on BAR mask registers */
  memset (&setRegs, 0, sizeof(setRegs));
  memset (&getRegs, 0, sizeof(getRegs));
  memset (&cmdStatus,  0, sizeof(cmdStatus));

  getRegs.cmdStatus = &cmdStatus;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Read CMD STATUS register failed!\n");
    return -4;
  }
  cmdStatus.dbi = 1;
  setRegs.cmdStatus = &cmdStatus;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET CMD STATUS register failed!\n");
    return -5;
  }
  
  /* Configure Masks*/
  memset (&setRegs, 0, sizeof(setRegs));
  type0Bar32bitIdx.reg.reg32 = PCIE_BAR_MASK;
  setRegs.type0Bar32bitIdx = &type0Bar32bitIdx;

  /* BAR 0 */
  type0Bar32bitIdx.idx = 0; /* configure BAR 0*/
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET BAR MASK register failed!\n");
    return -6;
  }
  
  /* BAR 1 */
  type0Bar32bitIdx.idx = 1; /* configure BAR 1*/
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET BAR MASK register failed!\n");
    return -7;
  }
  
  /* Disable writing on BAR Masks */
  memset (&setRegs, 0, sizeof(setRegs));
  memset (&getRegs, 0, sizeof(getRegs));
  memset (&cmdStatus,  0, sizeof(cmdStatus));

  getRegs.cmdStatus = &cmdStatus;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Read CMD STATUS register failed!\n");
    return -8;
  }
  cmdStatus.dbi = 0;
  setRegs.cmdStatus = &cmdStatus;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET CMD STATUS register failed!\n");
    return -9;
  }

  /* Enable memory access and mastership of the bus */
  memset (&setRegs, 0, sizeof(setRegs));
  memset (&getRegs, 0, sizeof(getRegs));

  getRegs.statusCmd = &statusCmd;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Read Status Comand register failed!\n");
    return -10;
  }
  statusCmd.memSp  = 1;
  statusCmd.busMs  = 1;
  statusCmd.resp   = 1;
  statusCmd.serrEn = 1;
  setRegs.statusCmd = &statusCmd;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET Status Command register failed!\n");
    return -11;
  }

  /* Enable Error Reporting */
  memset (&setRegs, 0, sizeof(setRegs));
  memset (&getRegs, 0, sizeof(getRegs));

  getRegs.devStatCtrl = &devStatCtrl;
  if ((retVal = Pcie_readRegs (handle, pcie_LOCATION_LOCAL, &getRegs)) != pcie_RET_OK) 
  {
    printf ("Regad Device Status Control register failed!\n");
    return -12;
  }
 
  devStatCtrl.reqRp = 1;
  devStatCtrl.fatalErRp = 1;
  devStatCtrl.nFatalErRp = 1;
  devStatCtrl.corErRp = 1;
  setRegs.devStatCtrl = &devStatCtrl;   
  
  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET Device Status Control register failed!\n");
    return -13;
  }

  /* Enable ECRC */
  memset (&setRegs, 0, sizeof(setRegs));
  
  accr.chkEn=1;
  accr.chkCap=1;
  accr.genEn=1;
  accr.genCap=1;
  setRegs.accr = &accr;

  if ((retVal = Pcie_writeRegs (handle, pcie_LOCATION_LOCAL, &setRegs)) != pcie_RET_OK) 
  {
    printf ("SET ACCR register failed!\n");
    return retVal;
  }

  return 0;
}
